@@ -1,20 +1,22 @@
 # TODO: token 相关的初始化可以通过 default Callable 来实现
-import hashlib
-from dataclasses import dataclass,field
+import logging
 import os
+import random
+import string
 import time
+from dataclasses import dataclass, field
 from datetime import datetime
 from pathlib import Path
-from typing import BinaryIO, Set, Dict, List, Optional, Union
-from urllib.parse import urlparse
-from botocore.exceptions import NoCredentialsError, PartialCredentialsError, ClientError
+from typing import BinaryIO, Dict, List, Optional, Set, Union
 
 import boto3
 import requests
+from botocore.exceptions import ClientError, NoCredentialsError, PartialCredentialsError
 
-from .dataset import Dataset, DatasetFile, UploadInfo, DatasetList, DatasetCommit
-import random
-import string
+from .dataset import Dataset, DatasetCommit, DatasetFile, DatasetList, UploadInfo
+
+# Create a logger for your library
+logger = logging.getLogger(__name__)
 
 HEYWHALE_SITE = os.getenv("HEYWHALE_HOST", "https://www.heywhale.com")
 HEYWHALE_DS_BUCKET = os.getenv("HEYWHALE_DS_BUCKET", "kesci")
@@ -22,12 +24,27 @@ HEYWHALE_DS_BUCKET = os.getenv("HEYWHALE_DS_BUCKET", "kesci")
 
 @dataclass(frozen=True)
 class DatasetFileForUpload:
+    """This is dataclass for file to upload.
+
+    Args:
+        local_path (str): local file path.
+        remote_path (str): remote file path in the dataset.
+
+    """
+
     local_path: str
     remote_path: str
 
 
 @dataclass
 class DatasetConstructor:
+    """DatasetConstructor
+    Args:
+        title (str): dataset title
+        short_description (str, optional): short description of the dataset. Defaults to "".
+        data_files (Set[DatasetFileForUpload], optional): dataset files. Defaults to set().
+    """
+
     title: str
     short_description: str
     data_files: Set[DatasetFileForUpload] = field(default_factory=set)
@@ -65,6 +82,18 @@ class DatasetConstructor:
                 self.add_file(file_path, path_in_dataset)
 
     def push_dataset(self) -> Dataset:
+        """
+        将数据集推送到平台并返回数据集对象。
+
+        Args:
+            无参数。
+
+        Returns:
+            Dataset: 推送到平台后的数据集对象。
+
+        Raises:
+            ValueError: 如果创建数据集失败，则抛出该异常。
+        """
         dataset = _create_dataset(
             title=self.title,
             files=list(self.data_files),
@@ -74,20 +103,34 @@ class DatasetConstructor:
             raise ValueError("Failed to create dataset")
         return dataset
 
+
 def parse_datetime(date_string: str) -> datetime:
     """
     Parse a datetime string into a datetime object.
 
-    :param date_string: The datetime string to parse.
-    :return: A datetime object.
+    Args:
+        date_string (str): The datetime string to parse.
+
+    Returns:
+        datetime: A datetime object.
+
     """
-    try:
-        return datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
-    except ValueError as e:
-        raise ValueError("Invalid datetime string format") from e
+    if date_string is None:
+        return datetime.now()
+    return datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
 
 
 def convert_to_dataset_file(files) -> List[DatasetFile]:
+    """
+    将给定的文件列表转换为 DatasetFile 对象的列表。
+
+    Args:
+        files (List[Dict]): 文件列表，每个元素为字典类型，包含 "_id"、"Token"、"Size" 和 "SubPath" 等字段。
+
+    Returns:
+        List[DatasetFile]: 转换后的 DatasetFile 对象列表。
+
+    """
     if files is None:
         return []
     dataset_files = [
@@ -106,10 +149,15 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
     """
     Fetches dataset details from Heywhale.
 
-    :param dataset_id: The ID of the dataset to fetch.
-    :param token: The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
-    :return: A Dataset object with the dataset details.
-    :raises ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
+    Args:
+        dataset_id (str): The ID of the dataset to fetch.
+        token (str, optional): The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
+
+    Returns:
+        Dataset: A Dataset object with the dataset details.
+
+    Raises:
+        ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
     if token is None:
         token = os.getenv("MW_TOKEN")
@@ -163,13 +211,13 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
             created_at=parse_datetime(response.json().get("CreateDate")),
             updated_at=parse_datetime(response.json().get("UpdateDate")),
         )
-    else:
-        response.raise_for_status()
-        return None
+
+    response.raise_for_status()
+    return None
 
 
 def _update_dataset(
-    id: str,
+    dataset_id: str,
     files: List[DatasetFile],
     commit_message: Optional[str] = None,
     token: Optional[str] = None,
@@ -188,8 +236,8 @@ def _update_dataset(
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    url = f"{HEYWHALE_SITE}/api/datasets/{id}/files"
-    headers = {"x-kesci-token": token, "x-kesci-resource": id}
+    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/files"
+    headers = {"x-kesci-token": token, "x-kesci-resource": dataset_id}
     data = {
         "Files": [file.key for file in files],
         "FilesStructure": [
@@ -215,9 +263,14 @@ def _get_update_token(token: Optional[str] = None) -> Optional[UploadInfo]:
     """
     Retrieves the upload token for updating a dataset.
 
-    :param token: The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
-    :return: An UploadInfo object containing the upload token details.
-    :raises ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
+    Args:
+        token: The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
+
+    Returns:
+        An UploadInfo object containing the upload token details.
+
+    Raises:
+        ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
     if token is None:
         token = os.getenv("MW_TOKEN")
@@ -305,19 +358,22 @@ def _upload_file(
 def upload_file(
     path_or_fileobj: Union[str, Path, bytes, BinaryIO],
     path_in_dataset: str | Path,
-    id: str | Dataset,
+    dataset_id: str,
     overwrite: bool = False,
     token: Optional[str] = None,
 ):
     """
     Uploads a file to a dataset.
 
-    :param path_or_fileobj: The path or file object of the file to upload.
-    :param path_in_dataset: The path to save the file in the dataset.
-    :param id: The ID of the dataset.
-    :param overwrite: Whether to overwrite an existing file with the same path in the dataset.
-    :param token: The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
-    :raises ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
+    Args:
+        path_or_fileobj (Union[str, Path, bytes, BinaryIO]): The path or file object of the file to upload.
+        path_in_dataset (str | Path): The path to save the file in the dataset.
+        id (str | Dataset): The ID of the dataset.
+        overwrite (bool, optional): Whether to overwrite an existing file with the same path in the dataset. Defaults to False.
+        token (Optional[str], optional): The token for authentication. If not provided, the function will use the 'MW_TOKEN'
+
+    Raises:
+        ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
     if token is None:
         token = os.getenv("MW_TOKEN")
@@ -325,14 +381,13 @@ def upload_file(
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    if isinstance(id, Dataset):
-        id = id._id
+
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
     object_key = _upload_file(path_or_fileobj, path_in_dataset, upload_info)
     new_dataset_files: List[DatasetFile] = []
-    dataset = get_dataset(id, token)
+    dataset = get_dataset(dataset_id, token)
     if dataset is not None:
         for file in dataset.files:
             if file.path_in_dataset() != path_in_dataset:
@@ -343,7 +398,10 @@ def upload_file(
         pp = str(Path(path_in_dataset).parent)
         new_dataset_files.append(DatasetFile("", object_key, 0, _norm_path(pp)))
     _update_dataset(
-        id, new_dataset_files, commit_message=f"添加文件 {path_in_dataset}", token=token
+        dataset_id,
+        new_dataset_files,
+        commit_message=f"添加文件 {path_in_dataset}",
+        token=token,
     )
 
 
@@ -372,27 +430,46 @@ def _norm_path(pp: str) -> str:
 def upload_folder(
     folder_path: str | Path,
     folder_in_dataset: str,
-    id: str | Dataset,
+    dataset_id: str,
     overwrite: bool = False,
     token: Optional[str] = None,
 ):
+    """
+    Upload a local folder to a dataset.
+
+    Args:
+        folder_path (str | Path): The path of the local folder to be uploaded.
+        folder_in_dataset (str): The path in the dataset where the folder will be uploaded.
+        dataset_id (str | Dataset): The ID or instance of the dataset to upload to.
+        overwrite (bool, optional): Whether to overwrite existing files in the dataset. Defaults to False.
+        token (Optional[str], optional): The authentication token for the dataset. If not provided, it will be retrieved from the 'MW_TOKEN' environment variable. Defaults to None.
+
+    Returns:
+        None
+
+    Raises:
+        ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set,
+                    or if the folder does not exist,
+                    or if the dataset is not found,
+                    or if failed to get upload token.
+    """
     if token is None:
         token = os.getenv("MW_TOKEN")
         if not token:
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    if isinstance(id, Dataset):
-        id = id._id
+    # check folder existence
+    if not os.path.isdir(folder_path):
+        raise ValueError("Folder does not exist.")
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
     new_dataset_files: Set[DatasetFile] = set()
-    dataset = get_dataset(id, token)
+    dataset = get_dataset(dataset_id, token)
     if dataset is None:
-        raise ValueError(f"Dataset '{id}' not found.")
-    # Walk through the directory
-    # print(f"folder path {folder_path}")
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
+
     for root, _dirs, files in os.walk(folder_path):
         # print("files")
         for file in files:
@@ -425,7 +502,7 @@ def upload_folder(
         return
     # print(new_dataset_files)
     _update_dataset(
-        id,
+        dataset_id,
         list(new_dataset_files),
         commit_message=f"添加目录 {folder_in_dataset}",
         token=token,
@@ -436,8 +513,11 @@ def _init_cache(cache_dir: Optional[str | Path] = None) -> Path:
     """
     Initializes the cache directory for the MW SDK.
 
-    :param cache_dir: The path to the cache directory. If not provided, the default cache directory will be used.
-    :return: The path to the cache directory.
+    Args:
+        cache_dir: The path to the cache directory. If not provided, the default cache directory will be used.
+
+    Returns:
+        The path to the cache directory.
     """
     if cache_dir is None:
         cache_dir = Path(os.getenv("MW_CACHE_DIR", "~/.cache/mw"))
@@ -462,11 +542,18 @@ def _download_single_file(
     """
     Downloads a single file from a dataset.
 
-    :param download_url: The URL to download the file from.
-    :param id: The ID of the dataset.
-    :param filename: The name of the file.
-    :param cache_dir: The cache directory.
-    :return: The path to the downloaded file.
+    Args:
+        download_url: The URL to download the file from.
+        id: The ID of the dataset.
+        filename: The name of the file.
+        etag: The ETag of the file.
+        commit: The commit id.
+        cache_dir: The cache directory.
+        local_dir: The local directory to save the file to. If not provided, the default cache directory will be used.
+        local_path: The path to save the file to. If specified, `local_dir` is ignored.
+
+    Returns:
+        The path to the downloaded file.
     """
     response = requests.get(download_url, stream=True, timeout=10)
     total_size = int(response.headers.get("content-length", 0))
@@ -618,7 +705,7 @@ def download_dir(
 
 
 def download_file(
-    id: str,
+    dataset_id: str,
     filename: str,
     commit: Optional[str] = None,
     cache_dir: Optional[str | Path] = None,
@@ -643,15 +730,15 @@ def download_file(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
     cache_dir = _init_cache(cache_dir)
-    dataset_detail = get_dataset(id)
+    dataset_detail = get_dataset(dataset_id)
     if dataset_detail is None:
-        raise ValueError(f"Dataset '{id}' not found.")
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
     path_to_key: Dict[str, str] = dict()
     for file in dataset_detail.files:
         path_to_key[file.path_in_dataset()] = file.key
 
     if filename not in path_to_key:
-        raise ValueError(f"File '{filename}' not found in dataset '{id}'.")
+        raise ValueError(f"File '{filename}' not found in dataset '{dataset_id}'.")
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
@@ -664,18 +751,18 @@ def download_file(
     s3 = session.client("s3", endpoint_url=upload_info.endpoint)
     bucket_name = upload_info.bucket
 
-    url = f"{HEYWHALE_SITE}/api/datasets/{id}/downloadUrl"
+    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/downloadUrl"
     headers = {
         "x-kesci-token": token,
     }
     if commit is None:
-        commit = dataset_detail.commits[-1]._id
+        commit = dataset_detail.commits[-1].get_id()
     params = {"Version": commit}
     response = requests.get(url, headers=headers, params=params, timeout=10)
     download_url: str = ""
     if response.status_code == 200:
         files = response.json().get("files")
-        # print(f"files {files}")
+        logger.debug("get dataset files through api: %s", files)
         for file in files:
             sub_path = file.get("SubPath") if file.get("SubPath") is not None else ""
             if os.path.join(sub_path, file.get("Name")) + file.get("Ext") == filename:
@@ -690,13 +777,11 @@ def download_file(
     # get etag
     try:
         response = s3.head_object(Bucket=bucket_name, Key=key)
-        # Extract the ETag from the response
         etag = response["ETag"]
-        # trim etag
         etag = etag[1:-1]
-        # print(f"Etag {etag}")
+        logger.debug("get object %s etag: %s", os.path.join(bucket_name, key), etag)
         slk_path = _download_single_file(
-            download_url, id, filename, etag, commit, cache_dir, local_dir
+            download_url, dataset_id, filename, etag, commit, cache_dir, local_dir
         )
     except NoCredentialsError:
         print("Error: AWS credentials not available.")
@@ -720,7 +805,7 @@ def download_file(
 
 
 def delete_file(
-    id: str,
+    dataset_id: str,
     filepath_in_dataset: str,
     token: Optional[str] = None,
 ):
@@ -731,7 +816,7 @@ def delete_file(
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    dataset = get_dataset(id, token)
+    dataset = get_dataset(dataset_id, token)
     if dataset is None:
         raise ValueError(f"Dataset '{id}' not found.")
     upload_info = _get_update_token(token)
@@ -750,7 +835,7 @@ def delete_file(
             print(f"delete file {file.path_in_dataset()}")
             continue
     _update_dataset(
-        id,
+        dataset_id,
         new_dataset_files,
         commit_message=f"删除文件 {filepath_in_dataset}",
         token=token,
@@ -763,7 +848,7 @@ def delete_folder():
 
 
 def create_commit(
-    id: str,
+    dataset_id: str,
     commit_message: str,
     token: Optional[str] = None,
 ) -> DatasetCommit:
@@ -776,13 +861,13 @@ def create_commit(
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
-    dataset = get_dataset(id, token)
+    dataset = get_dataset(dataset_id, token)
     if dataset is None:
-        raise ValueError(f"Dataset '{id}' not found.")
-    _update_dataset(id, dataset.files, commit_message, token)
-    dataset = get_dataset(id, token)
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
+    _update_dataset(dataset_id, dataset.files, commit_message, token)
+    dataset = get_dataset(dataset_id, token)
     if dataset is None:
-        raise ValueError(f"Dataset '{id}' not found.")
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
     return dataset.commits[-1]
 
 
@@ -792,6 +877,18 @@ def create_dataset(
     remote_dir: Union[str | None],
     short_description: Union[str | None],
 ) -> Optional[Dataset]:
+    """
+    根据提供的参数创建数据集对象
+
+    Args:
+        title (str): 数据集的标题
+        local_dir (str): 数据集在本地存储的目录
+        remote_dir (Union[str, None]): 数据集在远程存储的目录，可为空或None
+        short_description (Union[str, None]): 数据集的简短描述，可为空或None
+
+    Returns:
+        Optional[Dataset]: 创建的数据集对象，若创建失败则返回None
+    """
     if short_description is None:
         short_description = title
     dataset_constructor = DatasetConstructor(
@@ -800,7 +897,7 @@ def create_dataset(
     if remote_dir is None:
         remote_dir = ""
     dataset_constructor.add_dir(local_dir, remote_dir)
-    
+
     return _create_dataset(
         title=dataset_constructor.title,
         files=list(dataset_constructor.data_files),
@@ -861,7 +958,7 @@ def _create_dataset(
             {"Token": file.key, "SubPath": file.sub_path} for file in dataset_files
         ],
     }
-    #print(f"data {data}")
+    # print(f"data {data}")
     response = requests.post(
         url,
         json=data,
@@ -886,21 +983,34 @@ def _create_dataset(
 
 
 def delete_dataset(
-    id: str | Dataset,
+    dataset_id: str,
     token: Optional[str] = None,
 ):
+    """
+    删除指定的数据集。
+
+    Args:
+        id (str or Dataset): 数据集的ID或数据集对象。
+        token (Optional[str], optional): 访问数据集的token。如果未提供，则使用环境变量'MW_TOKEN'。默认为None。
+
+    Returns:
+        None
+
+    Raises:
+        ValueError: 如果未提供token且环境变量'MW_TOKEN'未设置。
+        HTTPError: 如果请求失败，则抛出HTTPError异常。
+
+    """
     if token is None:
         token = os.getenv("MW_TOKEN")
         if not token:
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    if isinstance(id, Dataset):
-        id = id._id
-    url = f"{HEYWHALE_SITE}/api/datasets/{id}"
+
+    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}"
     headers = {
         "x-kesci-token": token,
-        "x-kesci-resource": id,
     }
     response = requests.delete(
         url,
@@ -909,15 +1019,16 @@ def delete_dataset(
     )
     if response.status_code == 200:
         return
-    else:
-        print(response.text)
-        response.raise_for_status()
-        return
+
+    print(response.text)
+    response.raise_for_status()
+    return
 
 
-def list_datasets(
+def _list_datasets(
     title: str = "",
     limit: int = 10,
+    page: int = 1,
     start_date: time = None,
     end_date: time = None,
     token: Optional[str] = None,
@@ -928,20 +1039,23 @@ def list_datasets(
             raise ValueError(
                 "No token provided and 'MW_TOKEN' environment variable is not set."
             )
-    url = f"{HEYWHALE_SITE}/api/datasets"
     headers = {
         "x-kesci-token": token,
     }
+
+    url = f"{HEYWHALE_SITE}/api/user/datasets"
+    params = {
+        "perPage": limit,
+        "page": page,
+        "Title": title,
+        "startDate": start_date,
+        "endDate": end_date,
+    }
+
     response = requests.get(
         url,
         headers=headers,
-        params={
-            "perPage": limit,
-            "page": 0,
-            "Title": title,
-            "startDate": start_date,
-            "endDate": end_date,
-        },
+        params=params,
         timeout=10,
     )
     if response.status_code == 200:
@@ -984,5 +1098,93 @@ def list_datasets(
             limit=document.get("perPage"),
         )
     else:
+        print(response.text)
         response.raise_for_status()
         return None
+
+
+class DatasetIterator:
+    """
+    DatasetIterator 类
+
+    该类提供了一个迭代器接口，用于遍历数据集。
+    它封装了数据集加载、预处理和迭代的逻。
+    """
+
+    def __init__(self, title, page=1, limit=10):
+        self.title = title
+        self.limit = limit
+        self.page = page
+        self.total = 0
+        self.processed_count = 0
+        self.current_batch = []
+        self.finished = False
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        if not self.current_batch and not self.finished:
+            # Fetch the next page of datasets
+            dataset_list = _list_datasets(
+                title=self.title,
+                page=self.page,
+                limit=self.limit,
+            )
+            self.total = dataset_list.total
+
+            if not dataset_list.datasets:
+                self.finished = True
+                raise StopIteration
+
+            self.current_batch = dataset_list.datasets
+            self.processed_count += len(self.current_batch)
+            logger.debug("Processed: %d / Total: %d", self.processed_count, self.total)
+            # If all items have been processed, mark as finished
+            if self.processed_count >= self.total:
+                self.finished = True
+            else:
+                self.page += 1
+
+        if not self.current_batch:
+            raise StopIteration
+
+        # Return the next dataset in the current batch
+        return self.current_batch.pop(0)
+
+
+def list_all_datasets(title: str = "") -> List[Dataset]:
+    """
+    列出所有满足给定标题的数据集
+
+    Args:
+        title (str, optional): 数据集标题，默认为空字符串。
+
+    Returns:
+        List[Dataset]: 包含所有满足给定标题的数据集的列表。
+
+    """
+    datasets = list()
+    for dataset in DatasetIterator(title):
+        datasets.append(dataset)
+    return datasets
+
+
+def list_datasets(
+    title: str = "",
+    limit: int = 10,
+    page: int = 1,
+) -> DatasetIterator:
+    """
+    返回数据集的迭代器对象，可指定数据集标题、返回数据条数及页数
+
+    Args:
+        title (str, optional): 数据集标题. Defaults to "".
+        limit (int, optional): 每页返回的数据条数. Defaults to 10.
+        page (int, optional): 页码. Defaults to 1.
+
+    Returns:
+        DatasetIterator: 数据集的迭代器对象.
+
+    """
+    return DatasetIterator(title, page, limit)
